@@ -20,7 +20,7 @@
 
 /**
  * SECTION:element-oggmux
- * @see_also: <link linkend="gst-plugins-base-plugins-deoggmux">oggdemux</link>
+ * @see_also: <link linkend="gst-plugins-base-plugins-oggdemux">oggdemux</link>
  *
  * This element merges streams (audio and video) into ogg files.
  *
@@ -62,6 +62,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogg_mux_debug);
     : GST_BUFFER_TIMESTAMP (buf))
 
 #define GST_GP_FORMAT "[gp %8" G_GINT64_FORMAT "]"
+#define GST_GP_CAST(_gp) ((gint64) _gp)
 
 typedef enum
 {
@@ -70,13 +71,6 @@ typedef enum
 }
 GstOggFlag;
 
-/* elementfactory information */
-static const GstElementDetails gst_ogg_mux_details =
-GST_ELEMENT_DETAILS ("Ogg muxer",
-    "Codec/Muxer",
-    "mux ogg streams (info about ogg: http://xiph.org)",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* OggMux signals and args */
 enum
 {
@@ -175,7 +169,10 @@ gst_ogg_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
 
-  gst_element_class_set_details (element_class, &gst_ogg_mux_details);
+  gst_element_class_set_details_simple (element_class,
+      "Ogg muxer", "Codec/Muxer",
+      "mux ogg streams (info about ogg: http://xiph.org)",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -230,8 +227,6 @@ gst_ogg_mux_clear (GstOggMux * ogg_mux)
 {
   ogg_mux->pulling = NULL;
   ogg_mux->need_headers = TRUE;
-  ogg_mux->max_delay = DEFAULT_MAX_DELAY;
-  ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
   ogg_mux->delta_pad = NULL;
   ogg_mux->offset = 0;
   ogg_mux->next_ts = 0;
@@ -259,6 +254,9 @@ gst_ogg_mux_init (GstOggMux * ogg_mux)
       (GstCollectPadsFunction) GST_DEBUG_FUNCPTR (gst_ogg_mux_collected),
       ogg_mux);
 
+  ogg_mux->max_delay = DEFAULT_MAX_DELAY;
+  ogg_mux->max_page_delay = DEFAULT_MAX_PAGE_DELAY;
+
   gst_ogg_mux_clear (ogg_mux);
 }
 
@@ -280,7 +278,7 @@ gst_ogg_mux_finalize (GObject * object)
 static void
 gst_ogg_mux_ogg_pad_destroy_notify (GstCollectData * data)
 {
-  GstOggPad *oggpad = (GstOggPad *) data;
+  GstOggPadData *oggpad = (GstOggPadData *) data;
   GstBuffer *buf;
 
   ogg_stream_clear (&oggpad->stream);
@@ -312,7 +310,7 @@ static gboolean
 gst_ogg_mux_sink_event (GstPad * pad, GstEvent * event)
 {
   GstOggMux *ogg_mux = GST_OGG_MUX (gst_pad_get_parent (pad));
-  GstOggPad *ogg_pad = (GstOggPad *) gst_pad_get_element_private (pad);
+  GstOggPadData *ogg_pad = (GstOggPadData *) gst_pad_get_element_private (pad);
   gboolean ret;
 
   GST_DEBUG ("Got %s event on pad %s:%s", GST_EVENT_TYPE_NAME (event),
@@ -378,11 +376,11 @@ gst_ogg_mux_request_new_pad (GstElement * element,
     /* construct our own wrapper data structure for the pad to
      * keep track of its status */
     {
-      GstOggPad *oggpad;
+      GstOggPadData *oggpad;
 
-      oggpad = (GstOggPad *)
+      oggpad = (GstOggPadData *)
           gst_collect_pads_add_pad_full (ogg_mux->collect, newpad,
-          sizeof (GstOggPad), gst_ogg_mux_ogg_pad_destroy_notify);
+          sizeof (GstOggPadData), gst_ogg_mux_ogg_pad_destroy_notify);
       ogg_mux->active_pads++;
 
       oggpad->serial = serial;
@@ -475,7 +473,8 @@ gst_ogg_mux_buffer_from_page (GstOggMux * mux, ogg_page * page, gboolean delta)
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
 
   GST_LOG_OBJECT (mux, GST_GP_FORMAT
-      " created buffer %p from ogg page", ogg_page_granulepos (page), buffer);
+      " created buffer %p from ogg page",
+      GST_GP_CAST (ogg_page_granulepos (page)), buffer);
 
   return buffer;
 }
@@ -501,7 +500,8 @@ gst_ogg_mux_push_buffer (GstOggMux * mux, GstBuffer * buffer)
 
   caps = gst_pad_get_negotiated_caps (mux->srcpad);
   gst_buffer_set_caps (buffer, caps);
-  gst_caps_unref (caps);
+  if (caps)
+    gst_caps_unref (caps);
 
   return gst_pad_push (mux->srcpad, buffer);
 }
@@ -512,7 +512,7 @@ static gboolean
 gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 {
   GSList *walk;
-  GstOggPad *opad = NULL;       /* "oldest" pad */
+  GstOggPadData *opad = NULL;   /* "oldest" pad */
   GstClockTime oldest = GST_CLOCK_TIME_NONE;
   GstBuffer *buf = NULL;
   gboolean ret = FALSE;
@@ -521,7 +521,7 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
+    GstOggPadData *pad = (GstOggPadData *) walk->data;
 
     /* We need each queue to either be at EOS, or have one or more pages
      * available with a set granulepos (i.e. not -1), otherwise we don't have
@@ -562,7 +562,7 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad = (GstOggPad *) walk->data;
+    GstOggPadData *pad = (GstOggPadData *) walk->data;
 
     /* any page with a granulepos of -1 can be pushed immediately.
      * TODO: it CAN be, but it seems silly to do so? */
@@ -625,8 +625,8 @@ gst_ogg_mux_dequeue_page (GstOggMux * mux, GstFlowReturn * flowret)
  * counting.
  */
 static GstFlowReturn
-gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
-    gboolean delta)
+gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPadData * pad,
+    ogg_page * page, gboolean delta)
 {
   GstFlowReturn ret;
   GstBuffer *buffer = gst_ogg_mux_buffer_from_page (mux, page, delta);
@@ -644,7 +644,7 @@ gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
   GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
       " queued buffer page %p (gp time %"
       GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT
-      "), %d page buffers queued", ogg_page_granulepos (page),
+      "), %d page buffers queued", GST_GP_CAST (ogg_page_granulepos (page)),
       buffer, GST_TIME_ARGS (GST_BUFFER_OFFSET (buffer)),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       g_queue_get_length (pad->pagebuffers));
@@ -667,8 +667,8 @@ gst_ogg_mux_pad_queue_page (GstOggMux * mux, GstOggPad * pad, ogg_page * page,
  * of muxed pages
  */
 static gint
-gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
-    GstOggPad * second)
+gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPadData * first,
+    GstOggPadData * second)
 {
   guint64 firsttime, secondtime;
 
@@ -727,20 +727,20 @@ gst_ogg_mux_compare_pads (GstOggMux * ogg_mux, GstOggPad * first,
  * NULL when no pad was usable. "best" means the buffer marked
  * with the lowest timestamp. If best->buffer == NULL then nothing
  * should be done until more data arrives */
-static GstOggPad *
+static GstOggPadData *
 gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 {
-  GstOggPad *bestpad = NULL, *still_hungry = NULL;
+  GstOggPadData *bestpad = NULL, *still_hungry = NULL;
   GSList *walk;
 
   /* try to make sure we have a buffer from each usable pad first */
   walk = ogg_mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstCollectData *data;
 
     data = (GstCollectData *) walk->data;
-    pad = (GstOggPad *) data;
+    pad = (GstOggPadData *) data;
 
     walk = g_slist_next (walk);
 
@@ -836,18 +836,15 @@ gst_ogg_mux_queue_pads (GstOggMux * ogg_mux)
 }
 
 static GList *
-gst_ogg_mux_get_headers (GstOggPad * pad)
+gst_ogg_mux_get_headers (GstOggPadData * pad)
 {
   GList *res = NULL;
-  GstOggMux *ogg_mux;
   GstStructure *structure;
   GstCaps *caps;
   GstPad *thepad;
 
   thepad = pad->collect.pad;
 
-  ogg_mux = GST_OGG_MUX (GST_PAD_PARENT (thepad));
-
   GST_LOG_OBJECT (thepad, "getting headers");
 
   caps = gst_pad_get_negotiated_caps (thepad);
@@ -956,10 +953,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
 
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstPad *thepad;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
 
     walk = g_slist_next (walk);
@@ -977,7 +974,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
   GST_LOG_OBJECT (mux, "creating BOS pages");
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstBuffer *buf;
     ogg_packet packet;
     ogg_page page;
@@ -986,7 +983,7 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     GstStructure *structure;
     GstBuffer *hbuf;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
     caps = gst_pad_get_negotiated_caps (thepad);
     structure = gst_caps_get_structure (caps, 0);
@@ -1049,7 +1046,6 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     if (gst_structure_has_name (structure, "video/x-theora")) {
       GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Theora");
       hbufs = g_list_prepend (hbufs, hbuf);
-      pad->always_flush_page = TRUE;
     } else if (gst_structure_has_name (structure, "video/x-dirac")) {
       GST_DEBUG_OBJECT (thepad, "putting %s page at the front", "Dirac");
       hbufs = g_list_prepend (hbufs, hbuf);
@@ -1063,10 +1059,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
   GST_LOG_OBJECT (mux, "creating next headers");
   walk = mux->collect->data;
   while (walk) {
-    GstOggPad *pad;
+    GstOggPadData *pad;
     GstPad *thepad;
 
-    pad = (GstOggPad *) walk->data;
+    pad = (GstOggPadData *) walk->data;
     thepad = pad->collect.pad;
 
     walk = walk->next;
@@ -1133,15 +1129,16 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
     gst_caps_unref (caps);
   }
   /* and send the buffers */
-  hwalk = hbufs;
-  while (hwalk) {
-    GstBuffer *buf = GST_BUFFER (hwalk->data);
+  while (hbufs != NULL) {
+    GstBuffer *buf = GST_BUFFER (hbufs->data);
 
-    hwalk = hwalk->next;
+    hbufs = g_list_delete_link (hbufs, hbufs);
 
     if ((ret = gst_ogg_mux_push_buffer (mux, buf)) != GST_FLOW_OK)
       break;
   }
+  /* free any remaining nodes/buffers in case we couldn't push them */
+  g_list_foreach (hbufs, (GFunc) gst_mini_object_unref, NULL);
   g_list_free (hbufs);
 
   return ret;
@@ -1167,10 +1164,10 @@ gst_ogg_mux_send_headers (GstOggMux * mux)
  *    pads are at EOS)
  */
 static GstFlowReturn
-gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
+gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPadData * best)
 {
+  GstFlowReturn ret = GST_FLOW_OK;
   gboolean delta_unit;
-  GstFlowReturn ret;
   gint64 granulepos = 0;
   GstClockTime timestamp, gp_time;
 
@@ -1190,7 +1187,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
    * for the pad we were pulling from before */
   if (ogg_mux->pulling && best &&
       ogg_mux->pulling != best && ogg_mux->pulling->buffer) {
-    GstOggPad *pad = ogg_mux->pulling;
+    GstOggPadData *pad = ogg_mux->pulling;
 
     GstClockTime last_ts = GST_BUFFER_END_TIME (pad->buffer);
 
@@ -1202,7 +1199,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
       GST_LOG_OBJECT (pad->collect.pad,
           GST_GP_FORMAT " stored packet %" G_GINT64_FORMAT
           " will make page too long, flushing",
-          GST_BUFFER_OFFSET_END (pad->buffer), pad->stream.packetno);
+          GST_BUFFER_OFFSET_END (pad->buffer), (gint64) pad->stream.packetno);
 
       while (ogg_stream_flush (&pad->stream, &page)) {
         /* end time of this page is the timestamp of the next buffer */
@@ -1248,7 +1245,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     ogg_packet packet;
     ogg_page page;
     GstBuffer *buf, *tmpbuf;
-    GstOggPad *pad = ogg_mux->pulling;
+    GstOggPadData *pad = ogg_mux->pulling;
     gint64 duration;
     gboolean force_flush;
 
@@ -1284,7 +1281,8 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     packet.packetno = pad->packetno++;
     GST_LOG_OBJECT (pad->collect.pad, GST_GP_FORMAT
         " packet %" G_GINT64_FORMAT " (%ld bytes) created from buffer",
-        packet.granulepos, packet.packetno, packet.bytes);
+        GST_GP_CAST (packet.granulepos), (gint64) packet.packetno,
+        packet.bytes);
 
     packet.e_o_s = (pad->eos ? 1 : 0);
     tmpbuf = NULL;
@@ -1367,7 +1365,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     GST_LOG_OBJECT (pad->collect.pad,
         GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", gp time %"
         GST_TIME_FORMAT ", timestamp %" GST_TIME_FORMAT " packetin'd",
-        granulepos, packet.packetno, GST_TIME_ARGS (gp_time),
+        granulepos, (gint64) packet.packetno, GST_TIME_ARGS (gp_time),
         GST_TIME_ARGS (timestamp));
     /* don't need the old buffer anymore */
     gst_buffer_unref (pad->buffer);
@@ -1384,10 +1382,10 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
       GST_LOG_OBJECT (pad->collect.pad,
           GST_GP_FORMAT " packet %" G_GINT64_FORMAT ", time %"
           GST_TIME_FORMAT ") caused new page",
-          granulepos, packet.packetno, GST_TIME_ARGS (timestamp));
+          granulepos, (gint64) packet.packetno, GST_TIME_ARGS (timestamp));
       GST_LOG_OBJECT (pad->collect.pad,
-          GST_GP_FORMAT " new page %ld", ogg_page_granulepos (&page),
-          pad->stream.pageno);
+          GST_GP_FORMAT " new page %ld",
+          GST_GP_CAST (ogg_page_granulepos (&page)), pad->stream.pageno);
 
       if (ogg_page_granulepos (&page) == granulepos) {
         /* the packet we streamed in finishes on the current page,
@@ -1450,7 +1448,7 @@ gst_ogg_mux_process_best_pad (GstOggMux * ogg_mux, GstOggPad * best)
     }
   }
 
-  return GST_FLOW_OK;
+  return ret;
 }
 
 /* all_pads_eos:
@@ -1495,7 +1493,7 @@ beach:
 static GstFlowReturn
 gst_ogg_mux_collected (GstCollectPads * pads, GstOggMux * ogg_mux)
 {
-  GstOggPad *best;
+  GstOggPadData *best;
   GstFlowReturn ret;
   gint activebefore;
 
@@ -1591,7 +1589,7 @@ gst_ogg_mux_init_collectpads (GstCollectPads * collect)
 
   walk = collect->data;
   while (walk) {
-    GstOggPad *oggpad = (GstOggPad *) walk->data;
+    GstOggPadData *oggpad = (GstOggPadData *) walk->data;
 
     ogg_stream_init (&oggpad->stream, oggpad->serial);
     oggpad->packetno = 0;
@@ -1615,7 +1613,7 @@ gst_ogg_mux_clear_collectpads (GstCollectPads * collect)
   GSList *walk;
 
   for (walk = collect->data; walk; walk = g_slist_next (walk)) {
-    GstOggPad *oggpad = (GstOggPad *) walk->data;
+    GstOggPadData *oggpad = (GstOggPadData *) walk->data;
     GstBuffer *buf;
 
     ogg_stream_clear (&oggpad->stream);
