@@ -25,7 +25,7 @@
 #endif
 
 #include <gst/gst.h>
-#include <theora/theora.h>
+#include <theora/theoradec.h>
 #include <string.h>
 
 G_BEGIN_DECLS
@@ -58,21 +58,20 @@ struct _GstTheoraDec
   GstPad *srcpad;
 
   /* theora decoder state */
-  theora_state state;
-  theora_info info;
-  theora_comment comment;
+  th_dec_ctx *decoder;
+  //theora_state state;
+  th_setup_info *setup;
+  th_info info;
+  th_comment comment;
 
   gboolean have_header;
-  gboolean sent_newsegment;
-  gboolean is_old_bitstream;
-  guint64 granulepos;
-  guint64 granule_shift;
 
   GstClockTime last_timestamp;
-  guint64 frame_nr; /* unused */
+  guint64 frame_nr;
   gboolean need_keyframe;
   gint width, height;
   gint offset_x, offset_y;
+  gint output_bpp;
 
   gboolean crop;
 
@@ -83,16 +82,24 @@ struct _GstTheoraDec
   /* gather/decode queues for reverse playback */
   GList *gather;
   GList *decode;
+  GList *pendingevents;
 
   GstTagList *tags;
 
   /* segment info */ /* with STREAM_LOCK */
   GstSegment segment;
   gboolean discont;
+  guint32 seqnum;
 
   /* QoS stuff */ /* with LOCK*/
   gdouble proportion;
   GstClockTime earliest_time;
+  guint64 processed;
+  guint64 dropped;
+
+  gboolean have_par;
+  gint par_num;
+  gint par_den;
 };
 
 struct _GstTheoraDecClass
@@ -100,6 +107,8 @@ struct _GstTheoraDecClass
   GstElementClass parent_class;
 };
 
+GType gst_theora_dec_get_type (void);
+
 G_END_DECLS
 
 #endif /* __GST_THEORADEC_H__ */
