@@ -79,7 +79,7 @@ gst_property_probe_iface_init (GstPropertyProbeInterface * iface)
   if (!initialized) {
     /**
      * GstPropertyProbe::probe-needed
-     * %pspec: #GParamSpec that needs a probe
+     * @pspec: #GParamSpec that needs a probe
      *
      */
     /* FIXME:
@@ -165,7 +165,7 @@ gst_property_probe_get_property (GstPropertyProbe * probe, const gchar * name)
  * @probe: the #GstPropertyProbe to check.
  * @pspec: #GParamSpec of the property.
  *
- * Runs a probe on the property specified by %pspec
+ * Runs a probe on the property specified by @pspec
  */
 void
 gst_property_probe_probe_property (GstPropertyProbe * probe,
@@ -188,7 +188,7 @@ gst_property_probe_probe_property (GstPropertyProbe * probe,
  * @probe: the #GstPropertyProbe to check.
  * @name: name of the property.
  *
- * Runs a probe on the property specified by %name.
+ * Runs a probe on the property specified by @name.
  */
 void
 gst_property_probe_probe_property_name (GstPropertyProbe * probe,
@@ -339,14 +339,10 @@ GValueArray *
 gst_property_probe_probe_and_get_values (GstPropertyProbe * probe,
     const GParamSpec * pspec)
 {
-  GstPropertyProbeInterface *iface;
-
   g_return_val_if_fail (probe != NULL, NULL);
   g_return_val_if_fail (GST_IS_PROPERTY_PROBE (probe), NULL);
   g_return_val_if_fail (pspec != NULL, NULL);
 
-  iface = GST_PROPERTY_PROBE_GET_IFACE (probe);
-
   if (gst_property_probe_needs_probe (probe, pspec))
     gst_property_probe_probe_property (probe, pspec);
 
