@@ -219,6 +219,112 @@ error_overflow:
   return FALSE;
 }
 
+/**
+ * gst_video_format_parse_caps_interlaced:
+ * @caps: the fixed #GstCaps to parse
+ * @interlaced: whether @caps represents interlaced video or not, may be NULL (output)
+ *
+ * Extracts whether the caps represents interlaced content or not and places it
+ * in @interlaced.
+ *
+ * Since: 0.10.23
+ *
+ * Returns: TRUE if @caps was parsed correctly.
+ */
+gboolean
+gst_video_format_parse_caps_interlaced (GstCaps * caps, gboolean * interlaced)
+{
+  GstStructure *structure;
+
+  if (!gst_caps_is_fixed (caps))
+    return FALSE;
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (interlaced) {
+    if (!gst_structure_get_boolean (structure, "interlaced", interlaced))
+      *interlaced = FALSE;
+  }
+
+  return TRUE;
+}
+
+/**
+ * gst_video_parse_caps_color_matrix:
+ * @caps: the fixed #GstCaps to parse
+ *
+ * Extracts the color matrix used by the caps.  Possible values are
+ * "sdtv" for the standard definition color matrix (as specified in
+ * Rec. ITU-R BT.470-6) or "hdtv" for the high definition color
+ * matrix (as specified in Rec. ITU-R BT.709)
+ *
+ * Since: 0.10.29
+ *
+ * Returns: a color matrix string, or NULL if no color matrix could be
+ *     determined.
+ */
+const char *
+gst_video_parse_caps_color_matrix (GstCaps * caps)
+{
+  GstStructure *structure;
+  const char *s;
+
+  if (!gst_caps_is_fixed (caps))
+    return NULL;
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  s = gst_structure_get_string (structure, "color-matrix");
+  if (s)
+    return s;
+
+  if (gst_structure_has_name (structure, "video/x-raw-yuv")) {
+    return "sdtv";
+  }
+
+  return NULL;
+}
+
+/**
+ * gst_video_parse_caps_chroma_site:
+ * @caps: the fixed #GstCaps to parse
+ *
+ * Extracts the chroma site used by the caps.  Possible values are
+ * "mpeg2" for MPEG-2 style chroma siting (co-sited horizontally,
+ * halfway-sited vertically), "jpeg" for JPEG and Theora style
+ * chroma siting (halfway-sited both horizontally and vertically).
+ * Other chroma site values are possible, but uncommon.
+ * 
+ * When no chroma site is specified in the caps, it should be assumed
+ * to be "mpeg2".
+ *
+ * Since: 0.10.29
+ *
+ * Returns: a chroma site string, or NULL if no chroma site could be
+ *     determined.
+ */
+const char *
+gst_video_parse_caps_chroma_site (GstCaps * caps)
+{
+  GstStructure *structure;
+  const char *s;
+
+  if (!gst_caps_is_fixed (caps))
+    return NULL;
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  s = gst_structure_get_string (structure, "chroma-site");
+  if (s)
+    return s;
+
+  if (gst_structure_has_name (structure, "video/x-raw-yuv")) {
+    return "mpeg2";
+  }
+
+  return NULL;
+}
+
 /**
  * gst_video_format_parse_caps:
  * @caps: the #GstCaps to parse
@@ -298,6 +404,26 @@ gst_video_format_parse_caps (GstCaps * caps, GstVideoFormat * format,
       } else {
         ok = FALSE;
       }
+    } else if (gst_structure_has_name (structure, "video/x-raw-gray")) {
+      int depth;
+      int bpp;
+      int endianness;
+
+      ok &= gst_structure_get_int (structure, "depth", &depth);
+      ok &= gst_structure_get_int (structure, "bpp", &bpp);
+
+      if (bpp > 8)
+        ok &= gst_structure_get_int (structure, "endianness", &endianness);
+
+      if (depth == 8 && bpp == 8) {
+        *format = GST_VIDEO_FORMAT_GRAY8;
+      } else if (depth == 16 && bpp == 16 && endianness == G_BIG_ENDIAN) {
+        *format = GST_VIDEO_FORMAT_GRAY16_BE;
+      } else if (depth == 16 && bpp == 16 && endianness == G_LITTLE_ENDIAN) {
+        *format = GST_VIDEO_FORMAT_GRAY16_LE;
+      } else {
+        ok = FALSE;
+      }
     } else {
       ok = FALSE;
     }
@@ -314,6 +440,7 @@ gst_video_format_parse_caps (GstCaps * caps, GstVideoFormat * format,
   return ok;
 }
 
+
 /**
  * gst_video_parse_caps_framerate:
  * @caps: pointer to a #GstCaps instance
@@ -379,6 +506,39 @@ gst_video_parse_caps_pixel_aspect_ratio (GstCaps * caps, int *par_n, int *par_d)
   return TRUE;
 }
 
+/**
+ * gst_video_format_new_caps_interlaced:
+ * @format: the #GstVideoFormat describing the raw video format
+ * @width: width of video
+ * @height: height of video
+ * @framerate_n: numerator of frame rate
+ * @framerate_d: denominator of frame rate
+ * @par_n: numerator of pixel aspect ratio
+ * @par_d: denominator of pixel aspect ratio
+ * @interlaced: #TRUE if the format is interlaced
+ *
+ * Creates a new #GstCaps object based on the parameters provided.
+ *
+ * Since: 0.10.23
+ *
+ * Returns: a new #GstCaps object, or NULL if there was an error
+ */
+GstCaps *
+gst_video_format_new_caps_interlaced (GstVideoFormat format, int width,
+    int height, int framerate_n, int framerate_d, int par_n, int par_d,
+    gboolean interlaced)
+{
+  GstCaps *res;
+
+  res =
+      gst_video_format_new_caps (format, width, height, framerate_n,
+      framerate_d, par_n, par_d);
+  if (interlaced && (res != NULL))
+    gst_caps_set_simple (res, "interlaced", G_TYPE_BOOLEAN, TRUE, NULL);
+
+  return res;
+}
+
 /**
  * gst_video_format_new_caps:
  * @format: the #GstVideoFormat describing the raw video format
@@ -481,6 +641,53 @@ gst_video_format_new_caps (GstVideoFormat format, int width, int height,
     }
     return caps;
   }
+
+  if (gst_video_format_is_gray (format)) {
+    GstCaps *caps;
+    int bpp;
+    int depth;
+    int endianness;
+
+    switch (format) {
+      case GST_VIDEO_FORMAT_GRAY8:
+        bpp = depth = 8;
+        endianness = G_BIG_ENDIAN;
+        break;
+      case GST_VIDEO_FORMAT_GRAY16_BE:
+        bpp = depth = 16;
+        endianness = G_BIG_ENDIAN;
+        break;
+      case GST_VIDEO_FORMAT_GRAY16_LE:
+        bpp = depth = 16;
+        endianness = G_LITTLE_ENDIAN;
+        break;
+      default:
+        return NULL;
+        break;
+    }
+
+    if (bpp > 8) {
+      caps = gst_caps_new_simple ("video/x-raw-gray",
+          "bpp", G_TYPE_INT, bpp,
+          "depth", G_TYPE_INT, depth,
+          "width", G_TYPE_INT, width,
+          "height", G_TYPE_INT, height,
+          "framerate", GST_TYPE_FRACTION, framerate_n, framerate_d,
+          "pixel-aspect-ratio", GST_TYPE_FRACTION, par_n, par_d, NULL);
+    } else {
+      caps = gst_caps_new_simple ("video/x-raw-gray",
+          "bpp", G_TYPE_INT, bpp,
+          "depth", G_TYPE_INT, depth,
+          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
+          "width", G_TYPE_INT, width,
+          "height", G_TYPE_INT, height,
+          "framerate", GST_TYPE_FRACTION, framerate_n, framerate_d,
+          "pixel-aspect-ratio", GST_TYPE_FRACTION, par_n, par_d, NULL);
+    }
+
+    return caps;
+  }
+
   return NULL;
 }
 
@@ -506,6 +713,8 @@ gst_video_format_from_fourcc (guint32 fourcc)
       return GST_VIDEO_FORMAT_YV12;
     case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
       return GST_VIDEO_FORMAT_YUY2;
+    case GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U'):
+      return GST_VIDEO_FORMAT_YVYU;
     case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
       return GST_VIDEO_FORMAT_UYVY;
     case GST_MAKE_FOURCC ('A', 'Y', 'U', 'V'):
@@ -514,6 +723,16 @@ gst_video_format_from_fourcc (guint32 fourcc)
       return GST_VIDEO_FORMAT_Y41B;
     case GST_MAKE_FOURCC ('Y', '4', '2', 'B'):
       return GST_VIDEO_FORMAT_Y42B;
+    case GST_MAKE_FOURCC ('Y', '4', '4', '4'):
+      return GST_VIDEO_FORMAT_Y444;
+    case GST_MAKE_FOURCC ('v', '2', '1', '0'):
+      return GST_VIDEO_FORMAT_v210;
+    case GST_MAKE_FOURCC ('v', '2', '1', '6'):
+      return GST_VIDEO_FORMAT_v216;
+    case GST_MAKE_FOURCC ('N', 'V', '1', '2'):
+      return GST_VIDEO_FORMAT_NV12;
+    case GST_MAKE_FOURCC ('N', 'V', '2', '1'):
+      return GST_VIDEO_FORMAT_NV21;
     default:
       return GST_VIDEO_FORMAT_UNKNOWN;
   }
@@ -543,6 +762,8 @@ gst_video_format_to_fourcc (GstVideoFormat format)
       return GST_MAKE_FOURCC ('Y', 'V', '1', '2');
     case GST_VIDEO_FORMAT_YUY2:
       return GST_MAKE_FOURCC ('Y', 'U', 'Y', '2');
+    case GST_VIDEO_FORMAT_YVYU:
+      return GST_MAKE_FOURCC ('Y', 'V', 'Y', 'U');
     case GST_VIDEO_FORMAT_UYVY:
       return GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
     case GST_VIDEO_FORMAT_AYUV:
@@ -551,6 +772,16 @@ gst_video_format_to_fourcc (GstVideoFormat format)
       return GST_MAKE_FOURCC ('Y', '4', '1', 'B');
     case GST_VIDEO_FORMAT_Y42B:
       return GST_MAKE_FOURCC ('Y', '4', '2', 'B');
+    case GST_VIDEO_FORMAT_Y444:
+      return GST_MAKE_FOURCC ('Y', '4', '4', '4');
+    case GST_VIDEO_FORMAT_v210:
+      return GST_MAKE_FOURCC ('v', '2', '1', '0');
+    case GST_VIDEO_FORMAT_v216:
+      return GST_MAKE_FOURCC ('v', '2', '1', '6');
+    case GST_VIDEO_FORMAT_NV12:
+      return GST_MAKE_FOURCC ('N', 'V', '1', '2');
+    case GST_VIDEO_FORMAT_NV21:
+      return GST_MAKE_FOURCC ('N', 'V', '2', '1');
     default:
       return 0;
   }
@@ -646,10 +877,16 @@ gst_video_format_is_rgb (GstVideoFormat format)
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
+    case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_v210:
+    case GST_VIDEO_FORMAT_v216:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
       return FALSE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -684,10 +921,16 @@ gst_video_format_is_yuv (GstVideoFormat format)
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
+    case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_v210:
+    case GST_VIDEO_FORMAT_v216:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
       return TRUE;
     case GST_VIDEO_FORMAT_RGBx:
     case GST_VIDEO_FORMAT_BGRx:
@@ -705,6 +948,29 @@ gst_video_format_is_yuv (GstVideoFormat format)
   }
 }
 
+/**
+ * gst_video_format_is_gray:
+ * @format: a #GstVideoFormat
+ *
+ * Determine whether the video format is a grayscale format.
+ *
+ * Since: 0.10.29
+ *
+ * Returns: TRUE if @format represents grayscale video
+ */
+gboolean
+gst_video_format_is_gray (GstVideoFormat format)
+{
+  switch (format) {
+    case GST_VIDEO_FORMAT_GRAY8:
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
+      return TRUE;
+    default:
+      return FALSE;
+  }
+}
+
 /**
  * gst_video_format_has_alpha:
  * @format: a #GstVideoFormat
@@ -723,9 +989,15 @@ gst_video_format_has_alpha (GstVideoFormat format)
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
+    case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_v210:
+    case GST_VIDEO_FORMAT_v216:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
       return FALSE;
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBA:
@@ -780,6 +1052,7 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
         return GST_ROUND_UP_4 (GST_ROUND_UP_2 (width) / 2);
       }
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
       return GST_ROUND_UP_4 (width * 2);
     case GST_VIDEO_FORMAT_AYUV:
@@ -799,7 +1072,7 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
       if (component == 0) {
         return GST_ROUND_UP_4 (width);
       } else {
-        return GST_ROUND_UP_8 (width) / 4;
+        return GST_ROUND_UP_16 (width) / 4;
       }
     case GST_VIDEO_FORMAT_Y42B:
       if (component == 0) {
@@ -807,6 +1080,20 @@ gst_video_format_get_row_stride (GstVideoFormat format, int component,
       } else {
         return GST_ROUND_UP_8 (width) / 2;
       }
+    case GST_VIDEO_FORMAT_Y444:
+      return GST_ROUND_UP_4 (width);
+    case GST_VIDEO_FORMAT_v210:
+      return ((width + 47) / 48) * 128;
+    case GST_VIDEO_FORMAT_v216:
+      return GST_ROUND_UP_8 (width * 4);
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+      return GST_ROUND_UP_4 (width);
+    case GST_VIDEO_FORMAT_GRAY8:
+      return GST_ROUND_UP_4 (width);
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
+      return GST_ROUND_UP_4 (width * 2);
     default:
       return 0;
   }
@@ -837,8 +1124,10 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
+    case GST_VIDEO_FORMAT_Y444:
       return 1;
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
       if (component == 0) {
         return 2;
@@ -858,6 +1147,27 @@ gst_video_format_get_pixel_stride (GstVideoFormat format, int component)
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
       return 3;
+    case GST_VIDEO_FORMAT_v210:
+      /* v210 is packed at the bit level, so pixel stride doesn't make sense */
+      return 0;
+    case GST_VIDEO_FORMAT_v216:
+      if (component == 0) {
+        return 4;
+      } else {
+        return 8;
+      }
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+      if (component == 0) {
+        return 1;
+      } else {
+        return 2;
+      }
+    case GST_VIDEO_FORMAT_GRAY8:
+      return 1;
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
+      return 2;
     default:
       return 0;
   }
@@ -889,23 +1199,21 @@ gst_video_format_get_component_width (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
+    case GST_VIDEO_FORMAT_Y42B:
+    case GST_VIDEO_FORMAT_v210:
+    case GST_VIDEO_FORMAT_v216:
       if (component == 0) {
         return width;
       } else {
         return GST_ROUND_UP_2 (width) / 2;
       }
-    case GST_VIDEO_FORMAT_Y41B:        /* CHECKME: component_width for Y41B */
+    case GST_VIDEO_FORMAT_Y41B:
       if (component == 0) {
         return width;
       } else {
-        return GST_ROUND_UP_8 (width) / 4;
-      }
-    case GST_VIDEO_FORMAT_Y42B:        /* CHECKME: component_width for Y42B */
-      if (component == 0) {
-        return width;
-      } else {
-        return GST_ROUND_UP_8 (width) / 2;
+        return GST_ROUND_UP_4 (width) / 4;
       }
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBx:
@@ -918,6 +1226,12 @@ gst_video_format_get_component_width (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_ABGR:
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
+    case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+    case GST_VIDEO_FORMAT_GRAY8:
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
       return width;
     default:
       return 0;
@@ -949,6 +1263,8 @@ gst_video_format_get_component_height (GstVideoFormat format, int component,
   switch (format) {
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
       if (component == 0) {
         return height;
       } else {
@@ -957,6 +1273,7 @@ gst_video_format_get_component_height (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_Y41B:
     case GST_VIDEO_FORMAT_Y42B:
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
     case GST_VIDEO_FORMAT_AYUV:
     case GST_VIDEO_FORMAT_RGBx:
@@ -969,6 +1286,12 @@ gst_video_format_get_component_height (GstVideoFormat format, int component,
     case GST_VIDEO_FORMAT_ABGR:
     case GST_VIDEO_FORMAT_RGB:
     case GST_VIDEO_FORMAT_BGR:
+    case GST_VIDEO_FORMAT_Y444:
+    case GST_VIDEO_FORMAT_v210:
+    case GST_VIDEO_FORMAT_v216:
+    case GST_VIDEO_FORMAT_GRAY8:
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
       return height;
     default:
       return 0;
@@ -1032,6 +1355,14 @@ gst_video_format_get_component_offset (GstVideoFormat format, int component,
       if (component == 2)
         return 3;
       return 0;
+    case GST_VIDEO_FORMAT_YVYU:
+      if (component == 0)
+        return 0;
+      if (component == 1)
+        return 3;
+      if (component == 2)
+        return 1;
+      return 0;
     case GST_VIDEO_FORMAT_UYVY:
       if (component == 0)
         return 1;
@@ -1116,7 +1447,8 @@ gst_video_format_get_component_offset (GstVideoFormat format, int component,
       if (component == 1)
         return GST_ROUND_UP_4 (width) * height;
       if (component == 2)
-        return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 4)) * height;
+        return (GST_ROUND_UP_4 (width) +
+            (GST_ROUND_UP_16 (width) / 4)) * height;
       return 0;
     case GST_VIDEO_FORMAT_Y42B:
       if (component == 0)
@@ -1126,6 +1458,37 @@ gst_video_format_get_component_offset (GstVideoFormat format, int component,
       if (component == 2)
         return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 2)) * height;
       return 0;
+    case GST_VIDEO_FORMAT_Y444:
+      return GST_ROUND_UP_4 (width) * height * component;
+    case GST_VIDEO_FORMAT_v210:
+      /* v210 is bit-packed, so this doesn't make sense */
+      return 0;
+    case GST_VIDEO_FORMAT_v216:
+      if (component == 0)
+        return 0;
+      if (component == 1)
+        return 2;
+      if (component == 2)
+        return 6;
+      return 0;
+    case GST_VIDEO_FORMAT_NV12:
+      if (component == 0)
+        return 0;
+      if (component == 1)
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
+      if (component == 2)
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) + 1;
+    case GST_VIDEO_FORMAT_NV21:
+      if (component == 0)
+        return 0;
+      if (component == 1)
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) + 1;
+      if (component == 2)
+        return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height);
+    case GST_VIDEO_FORMAT_GRAY8:
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
+      return 0;
     default:
       return 0;
   }
@@ -1160,6 +1523,7 @@ gst_video_format_get_size (GstVideoFormat format, int width, int height)
           (GST_ROUND_UP_2 (height) / 2) * 2;
       return size;
     case GST_VIDEO_FORMAT_YUY2:
+    case GST_VIDEO_FORMAT_YVYU:
     case GST_VIDEO_FORMAT_UYVY:
       return GST_ROUND_UP_4 (width * 2) * height;
     case GST_VIDEO_FORMAT_AYUV:
@@ -1176,11 +1540,25 @@ gst_video_format_get_size (GstVideoFormat format, int width, int height)
     case GST_VIDEO_FORMAT_BGR:
       return GST_ROUND_UP_4 (width * 3) * height;
     case GST_VIDEO_FORMAT_Y41B:
-      /* simplification of ROUNDUP4(w)*h + 2*((ROUNDUP8(w)/4)*h */
-      return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_8 (width) / 2)) * height;
+      /* simplification of ROUNDUP4(w)*h + 2*((ROUNDUP16(w)/4)*h */
+      return (GST_ROUND_UP_4 (width) + (GST_ROUND_UP_16 (width) / 2)) * height;
     case GST_VIDEO_FORMAT_Y42B:
-      /* simplification of ROUNDUP4(w)*h + 2*(ROUNDUP8(w)/2)*h: */
+      /* simplification of ROUNDUP4(w)*h + 2*(ROUNDUP8(w)/2)*h */
       return (GST_ROUND_UP_4 (width) + GST_ROUND_UP_8 (width)) * height;
+    case GST_VIDEO_FORMAT_Y444:
+      return GST_ROUND_UP_4 (width) * height * 3;
+    case GST_VIDEO_FORMAT_v210:
+      return ((width + 47) / 48) * 128 * height;
+    case GST_VIDEO_FORMAT_v216:
+      return GST_ROUND_UP_8 (width * 4) * height;
+    case GST_VIDEO_FORMAT_NV12:
+    case GST_VIDEO_FORMAT_NV21:
+      return GST_ROUND_UP_4 (width) * GST_ROUND_UP_2 (height) * 3 / 2;
+    case GST_VIDEO_FORMAT_GRAY8:
+      return GST_ROUND_UP_4 (width) * height;
+    case GST_VIDEO_FORMAT_GRAY16_BE:
+    case GST_VIDEO_FORMAT_GRAY16_LE:
+      return GST_ROUND_UP_4 (width * 2) * height;
     default:
       return 0;
   }
@@ -1313,3 +1691,71 @@ done:
 
   return ret;
 }
+
+#define GST_VIDEO_EVENT_STILL_STATE_NAME "GstEventStillFrame"
+
+/**
+ * gst_video_event_new_still_frame:
+ * @in_still: boolean value for the still-frame state of the event.
+ *
+ * Creates a new Still Frame event. If @in_still is %TRUE, then the event
+ * represents the start of a still frame sequence. If it is %FALSE, then
+ * the event ends a still frame sequence.
+ *
+ * To parse an event created by gst_video_event_new_still_frame() use
+ * gst_video_event_parse_still_frame().
+ *
+ * Returns: The new GstEvent
+ * Since: 0.10.26
+ */
+GstEvent *
+gst_video_event_new_still_frame (gboolean in_still)
+{
+  GstEvent *still_event;
+  GstStructure *s;
+
+  s = gst_structure_new (GST_VIDEO_EVENT_STILL_STATE_NAME,
+      "still-state", G_TYPE_BOOLEAN, in_still, NULL);
+  still_event = gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, s);
+
+  return still_event;
+}
+
+/**
+ * gst_video_event_parse_still_frame:
+ * @event: A #GstEvent to parse
+ * @in_still: A boolean to receive the still-frame status from the event, or NULL
+ *
+ * Parse a #GstEvent, identify if it is a Still Frame event, and
+ * return the still-frame state from the event if it is.
+ * If the event represents the start of a still frame, the in_still
+ * variable will be set to TRUE, otherwise FALSE. It is OK to pass NULL for the
+ * in_still variable order to just check whether the event is a valid still-frame
+ * event.
+ *
+ * Create a still frame event using gst_video_event_new_still_frame()
+ *
+ * Returns: %TRUE if the event is a valid still-frame event. %FALSE if not
+ * Since: 0.10.26
+ */
+gboolean
+gst_video_event_parse_still_frame (GstEvent * event, gboolean * in_still)
+{
+  const GstStructure *s;
+  gboolean ev_still_state;
+
+  g_return_val_if_fail (event != NULL, FALSE);
+
+  if (GST_EVENT_TYPE (event) != GST_EVENT_CUSTOM_DOWNSTREAM)
+    return FALSE;               /* Not a still frame event */
+
+  s = gst_event_get_structure (event);
+  if (s == NULL
+      || !gst_structure_has_name (s, GST_VIDEO_EVENT_STILL_STATE_NAME))
+    return FALSE;               /* Not a still frame event */
+  if (!gst_structure_get_boolean (s, "still-state", &ev_still_state))
+    return FALSE;               /* Not a still frame event */
+  if (in_still)
+    *in_still = ev_still_state;
+  return TRUE;
+}
