@@ -30,6 +30,9 @@
 #include "avcodec.h"
 #include "gstffmpegcodecmap.h"
 
+GST_DEBUG_CATEGORY_EXTERN (ffmpegcolorspace_debug);
+#define GST_CAT_DEFAULT ffmpegcolorspace_debug
+
 static GstCaps *
 gst_ff_vid_caps_new (AVCodecContext * context,
     const char *mimetype, const char *fieldname, ...)
@@ -801,7 +804,8 @@ gst_ffmpegcsp_caps_with_codectype (enum CodecType type,
  */
 int
 gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
-    uint8_t * ptr, enum PixelFormat pix_fmt, int width, int height)
+    uint8_t * ptr, enum PixelFormat pix_fmt, int width, int height,
+    int interlaced)
 {
   int size, w2, h2, size2;
   int stride, stride2;
@@ -809,6 +813,8 @@ gst_ffmpegcsp_avpicture_fill (AVPicture * picture,
 
   pinfo = get_pix_fmt_info (pix_fmt);
 
+  picture->interlaced = interlaced;
+
   switch (pix_fmt) {
     case PIX_FMT_YUV420P:
     case PIX_FMT_YUV422P:
