@@ -46,12 +46,6 @@
 GST_DEBUG_CATEGORY_STATIC (video_test_src_debug);
 #define GST_CAT_DEFAULT video_test_src_debug
 
-static const GstElementDetails video_test_src_details =
-GST_ELEMENT_DETAILS ("Video test source",
-    "Source/Video",
-    "Creates a test video stream",
-    "David A. Schleef <ds@schleef.org>");
-
 #define DEFAULT_PATTERN            GST_VIDEO_TEST_SRC_SMPTE
 #define DEFAULT_TIMESTAMP_OFFSET   0
 #define DEFAULT_IS_LIVE            FALSE
@@ -129,6 +123,9 @@ gst_video_test_src_pattern_get_type (void)
     {GST_VIDEO_TEST_SRC_BLINK, "Blink", "blink"},
     {GST_VIDEO_TEST_SRC_SMPTE75, "SMPTE 75% color bars", "smpte75"},
     {GST_VIDEO_TEST_SRC_ZONE_PLATE, "Zone plate", "zone-plate"},
+    {GST_VIDEO_TEST_SRC_GAMUT, "Gamut checkers", "gamut"},
+    {GST_VIDEO_TEST_SRC_CHROMA_ZONE_PLATE, "Chroma zone plate",
+        "chroma-zone-plate"},
     {0, NULL, NULL}
   };
 
@@ -162,7 +159,9 @@ gst_video_test_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &video_test_src_details);
+  gst_element_class_set_details_simple (element_class,
+      "Video test source", "Source/Video",
+      "Creates a test video stream", "David A. Schleef <ds@schleef.org>");
 
   gst_element_class_add_pad_template (element_class,
       gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
@@ -354,6 +353,12 @@ gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     case GST_VIDEO_TEST_SRC_ZONE_PLATE:
       videotestsrc->make_image = gst_video_test_src_zoneplate;
       break;
+    case GST_VIDEO_TEST_SRC_GAMUT:
+      videotestsrc->make_image = gst_video_test_src_gamut;
+      break;
+    case GST_VIDEO_TEST_SRC_CHROMA_ZONE_PLATE:
+      videotestsrc->make_image = gst_video_test_src_chromazoneplate;
+      break;
     default:
       g_assert_not_reached ();
   }
@@ -491,9 +496,6 @@ static GstCaps *
 gst_video_test_src_getcaps (GstBaseSrc * bsrc)
 {
   static GstCaps *capslist = NULL;
-  GstVideoTestSrc *videotestsrc;
-
-  videotestsrc = GST_VIDEO_TEST_SRC (bsrc);
 
   if (!capslist) {
     GstCaps *caps;
@@ -519,11 +521,12 @@ gst_video_test_src_getcaps (GstBaseSrc * bsrc)
 static gboolean
 gst_video_test_src_parse_caps (const GstCaps * caps,
     gint * width, gint * height, gint * rate_numerator, gint * rate_denominator,
-    struct fourcc_list_struct **fourcc)
+    struct fourcc_list_struct **fourcc, GstVideoTestSrcColorSpec * color_spec)
 {
   const GstStructure *structure;
   GstPadLinkReturn ret;
   const GValue *framerate;
+  const char *csp;
 
   GST_DEBUG ("parsing caps");
 
@@ -545,6 +548,20 @@ gst_video_test_src_parse_caps (const GstCaps * caps,
   } else
     goto no_framerate;
 
+  csp = gst_structure_get_string (structure, "color-matrix");
+  if (csp) {
+    if (strcmp (csp, "sdtv") == 0) {
+      *color_spec = GST_VIDEO_TEST_SRC_BT601;
+    } else if (strcmp (csp, "hdtv") == 0) {
+      *color_spec = GST_VIDEO_TEST_SRC_BT709;
+    } else {
+      GST_DEBUG ("unknown color-matrix");
+      return FALSE;
+    }
+  } else {
+    *color_spec = GST_VIDEO_TEST_SRC_BT601;
+  }
+
   return ret;
 
   /* ERRORS */
@@ -567,11 +584,12 @@ gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
   gint width, height, rate_denominator, rate_numerator;
   struct fourcc_list_struct *fourcc;
   GstVideoTestSrc *videotestsrc;
+  GstVideoTestSrcColorSpec color_spec;
 
   videotestsrc = GST_VIDEO_TEST_SRC (bsrc);
 
   res = gst_video_test_src_parse_caps (caps, &width, &height,
-      &rate_numerator, &rate_denominator, &fourcc);
+      &rate_numerator, &rate_denominator, &fourcc, &color_spec);
   if (res) {
     /* looks ok here */
     videotestsrc->fourcc = fourcc;
@@ -580,6 +598,7 @@ gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
     videotestsrc->rate_numerator = rate_numerator;
     videotestsrc->rate_denominator = rate_denominator;
     videotestsrc->bpp = videotestsrc->fourcc->bitspp;
+    videotestsrc->color_spec = color_spec;
 
     GST_DEBUG_OBJECT (videotestsrc, "size %dx%d, %d/%d fps",
         videotestsrc->width, videotestsrc->height,
